@@ -1,6 +1,8 @@
 /*
  * Copyright (C) 2011 Colin Walters <walters@verbum.org>
  *
+ * SPDX-License-Identifier: LGPL-2.0+
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -26,10 +28,8 @@
 #include "ostree-core.h"
 #include "ostree-types.h"
 #include "ostree-async-progress.h"
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
 #include "ostree-ref.h"
 #include "ostree-repo-finder.h"
-#endif
 #include "ostree-sepolicy.h"
 #include "ostree-gpg-verify-result.h"
 
@@ -105,50 +105,6 @@ OstreeRepo *  ostree_repo_create_at (int             dfd,
                                      GCancellable   *cancellable,
                                      GError        **error);
 
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
-
-/**
- * OstreeRepoLockType:
- * @OSTREE_REPO_LOCK_SHARED: A shared lock
- * @OSTREE_REPO_LOCK_EXCLUSIVE: An exclusive lock
- *
- * The type of repository lock to acquire.
- *
- * Since: 2017.14
- */
-typedef enum {
-  OSTREE_REPO_LOCK_SHARED,
-  OSTREE_REPO_LOCK_EXCLUSIVE
-} OstreeRepoLockType;
-
-_OSTREE_PUBLIC
-gboolean      ostree_repo_lock_push (OstreeRepo          *self,
-                                     OstreeRepoLockType   lock_type,
-                                     GCancellable        *cancellable,
-                                     GError             **error);
-_OSTREE_PUBLIC
-gboolean      ostree_repo_lock_pop (OstreeRepo    *self,
-                                    GCancellable  *cancellable,
-                                    GError       **error);
-
-/**
- * OstreeRepoAutoLock: (skip)
- *
- * This is simply an alias to #OstreeRepo used for automatic lock cleanup.
- * See ostree_repo_auto_lock_push() for its intended usage.
- *
- * Since: 2017.14
- */
-typedef OstreeRepo OstreeRepoAutoLock;
-
-_OSTREE_PUBLIC
-OstreeRepoAutoLock * ostree_repo_auto_lock_push (OstreeRepo          *self,
-                                                 OstreeRepoLockType   lock_type,
-                                                 GCancellable        *cancellable,
-                                                 GError             **error);
-_OSTREE_PUBLIC
-void          ostree_repo_auto_lock_cleanup (OstreeRepoAutoLock *lock);
-
 _OSTREE_PUBLIC
 const gchar * ostree_repo_get_collection_id (OstreeRepo   *self);
 _OSTREE_PUBLIC
@@ -156,7 +112,8 @@ gboolean      ostree_repo_set_collection_id (OstreeRepo   *self,
                                              const gchar  *collection_id,
                                              GError      **error);
 
-#endif  /* OSTREE_ENABLE_EXPERIMENTAL_API */
+_OSTREE_PUBLIC
+const gchar * const * ostree_repo_get_default_repo_finders (OstreeRepo   *self);
 
 _OSTREE_PUBLIC
 GFile *       ostree_repo_get_path (OstreeRepo  *self);
@@ -173,6 +130,10 @@ gboolean      ostree_repo_equal (OstreeRepo *a,
 _OSTREE_PUBLIC
 OstreeRepoMode ostree_repo_get_mode (OstreeRepo  *self);
 
+_OSTREE_PUBLIC
+gboolean       ostree_repo_get_min_free_space_bytes (OstreeRepo *self,
+                                                     guint64 *out_reserved_bytes,
+                                                     GError **error);
 _OSTREE_PUBLIC
 GKeyFile *    ostree_repo_get_config (OstreeRepo *self);
 
@@ -379,15 +340,11 @@ void          ostree_repo_transaction_set_ref     (OstreeRepo *self,
                                                    const char *ref,
                                                    const char *checksum);
 
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
-
 _OSTREE_PUBLIC
 void          ostree_repo_transaction_set_collection_ref (OstreeRepo                *self,
                                                           const OstreeCollectionRef *ref,
                                                           const char                *checksum);
 
-#endif  /* OSTREE_ENABLE_EXPERIMENTAL_API */
-
 _OSTREE_PUBLIC
 gboolean      ostree_repo_set_ref_immediate (OstreeRepo *self,
                                              const char *remote,
@@ -404,8 +361,6 @@ gboolean      ostree_repo_set_alias_ref_immediate (OstreeRepo *self,
                                                    GCancellable  *cancellable,
                                                    GError       **error);
 
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
-
 _OSTREE_PUBLIC
 gboolean      ostree_repo_set_collection_ref_immediate (OstreeRepo                 *self,
                                                         const OstreeCollectionRef  *ref,
@@ -413,8 +368,6 @@ gboolean      ostree_repo_set_collection_ref_immediate (OstreeRepo
                                                         GCancellable               *cancellable,
                                                         GError                    **error);
 
-#endif  /* OSTREE_ENABLE_EXPERIMENTAL_API */
-
 _OSTREE_PUBLIC
 gboolean      ostree_repo_has_object (OstreeRepo           *self,
                                       OstreeObjectType      objtype,
@@ -519,7 +472,6 @@ gboolean      ostree_repo_resolve_rev_ext (OstreeRepo                    *self,
                                            char                         **out_rev,
                                            GError                       **error);
 
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
 _OSTREE_PUBLIC
 gboolean      ostree_repo_resolve_collection_ref (OstreeRepo                    *self,
                                                   const OstreeCollectionRef     *ref,
@@ -528,7 +480,6 @@ gboolean      ostree_repo_resolve_collection_ref (OstreeRepo
                                                   char                         **out_rev,
                                                   GCancellable                  *cancellable,
                                                   GError                       **error);
-#endif  /* OSTREE_ENABLE_EXPERIMENTAL_API */
 
 _OSTREE_PUBLIC
 gboolean      ostree_repo_list_refs (OstreeRepo       *self,
@@ -564,14 +515,12 @@ gboolean ostree_repo_remote_list_refs (OstreeRepo       *self,
                                        GCancellable     *cancellable,
                                        GError          **error);
 
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
 _OSTREE_PUBLIC
 gboolean ostree_repo_remote_list_collection_refs (OstreeRepo    *self,
                                                   const char    *remote_name,
                                                   GHashTable   **out_all_refs,
                                                   GCancellable  *cancellable,
                                                   GError       **error);
-#endif  /* OSTREE_ENABLE_EXPERIMENTAL_API */
 
 _OSTREE_PUBLIC
 gboolean      ostree_repo_load_variant (OstreeRepo  *self,
@@ -940,6 +889,34 @@ ostree_repo_checkout_tree (OstreeRepo               *self,
                            GCancellable             *cancellable,
                            GError                  **error);
 
+/**
+ * OstreeRepoCheckoutFilterResult:
+ * @OSTREE_REPO_CHECKOUT_FILTER_ALLOW: Do checkout this object
+ * @OSTREE_REPO_CHECKOUT_FILTER_SKIP: Ignore this object
+ *
+ * Since: 2018.2
+ */
+typedef enum {
+  OSTREE_REPO_CHECKOUT_FILTER_ALLOW,
+  OSTREE_REPO_CHECKOUT_FILTER_SKIP
+} OstreeRepoCheckoutFilterResult;
+
+/**
+ * OstreeRepoCheckoutFilter:
+ * @repo: Repo
+ * @path: Path to file
+ * @stbuf: File information
+ * @user_data: User data
+ *
+ * Returns: #OstreeRepoCheckoutFilterResult saying whether or not to checkout this file
+ *
+ * Since: 2018.2
+ */
+typedef OstreeRepoCheckoutFilterResult (*OstreeRepoCheckoutFilter) (OstreeRepo    *repo,
+                                                                    const char    *path,
+                                                                    struct stat   *stbuf,
+                                                                    gpointer       user_data);
+
 /**
  * OstreeRepoCheckoutAtOptions:
  *
@@ -959,7 +936,8 @@ typedef struct {
   gboolean no_copy_fallback;
   gboolean force_copy; /* Since: 2017.6 */
   gboolean bareuseronly_dirs; /* Since: 2017.7 */
-  gboolean unused_bools[5];
+  gboolean force_copy_zerosized; /* Since: 2018.9 */
+  gboolean unused_bools[4];
   /* 4 byte hole on 64 bit */
 
   const char *subpath;
@@ -967,7 +945,9 @@ typedef struct {
   OstreeRepoDevInoCache *devino_to_csum_cache;
 
   int unused_ints[6];
-  gpointer unused_ptrs[5];
+  gpointer unused_ptrs[3];
+  OstreeRepoCheckoutFilter filter; /* Since: 2018.2 */
+  gpointer filter_user_data; /* Since: 2018.2 */
   OstreeSePolicy *sepolicy; /* Since: 2017.6 */
   const char *sepolicy_prefix;
 } OstreeRepoCheckoutAtOptions;
@@ -1080,6 +1060,12 @@ gboolean ostree_repo_static_delta_execute_offline (OstreeRepo
 _OSTREE_PUBLIC
 GHashTable *ostree_repo_traverse_new_reachable (void);
 
+_OSTREE_PUBLIC
+GHashTable *ostree_repo_traverse_new_parents (void);
+
+_OSTREE_PUBLIC
+char ** ostree_repo_traverse_parents_get_commits (GHashTable *parents, GVariant *object);
+
 _OSTREE_PUBLIC
 gboolean ostree_repo_traverse_commit (OstreeRepo         *repo,
                                       const char         *commit_checksum,
@@ -1095,6 +1081,14 @@ gboolean ostree_repo_traverse_commit_union (OstreeRepo         *repo,
                                             GHashTable         *inout_reachable,
                                             GCancellable       *cancellable,
                                             GError            **error);
+_OSTREE_PUBLIC
+gboolean ostree_repo_traverse_commit_union_with_parents (OstreeRepo         *repo,
+                                                         const char         *commit_checksum,
+                                                         int                 maxdepth,
+                                                         GHashTable         *inout_reachable,
+                                                         GHashTable         *inout_parents,
+                                                         GCancellable       *cancellable,
+                                                         GError            **error);
 
 struct _OstreeRepoCommitTraverseIter {
   gboolean initialized;
@@ -1198,6 +1192,14 @@ struct _OstreeRepoPruneOptions {
 
 typedef struct _OstreeRepoPruneOptions OstreeRepoPruneOptions;
 
+_OSTREE_PUBLIC
+gboolean
+ostree_repo_traverse_reachable_refs (OstreeRepo *self,
+                                     guint       depth,
+                                     GHashTable *reachable,
+                                     GCancellable *cancellable,
+                                     GError      **error);
+
 _OSTREE_PUBLIC
 gboolean ostree_repo_prune_from_reachable (OstreeRepo             *self,
                                            OstreeRepoPruneOptions *options,
@@ -1253,8 +1255,6 @@ gboolean ostree_repo_pull_with_options (OstreeRepo             *self,
                                         GCancellable           *cancellable,
                                         GError                **error);
 
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
-
 _OSTREE_PUBLIC
 void ostree_repo_find_remotes_async (OstreeRepo                         *self,
                                      const OstreeCollectionRef * const  *refs,
@@ -1296,8 +1296,6 @@ gboolean ostree_repo_list_collection_refs (OstreeRepo                 *self,
                                            GCancellable               *cancellable,
                                            GError                     **error);
 
-#endif /* OSTREE_ENABLE_EXPERIMENTAL_API */
-
 _OSTREE_PUBLIC
 void ostree_repo_pull_default_console_progress_changed (OstreeAsyncProgress *progress,
                                                         gpointer             user_data);
@@ -1382,8 +1380,6 @@ gboolean ostree_repo_regenerate_summary (OstreeRepo     *self,
                                          GCancellable   *cancellable,
                                          GError        **error);
 
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
-
 /**
  * OSTREE_REPO_METADATA_REF:
  *
@@ -1403,11 +1399,32 @@ gboolean ostree_repo_regenerate_summary (OstreeRepo     *self,
  * keys must be namespaced by product or developer. For example,
  * `exampleos.end-of-life`. The `ostree.` prefix is reserved.
  *
- * Since: 2017.8
+ * Since: 2018.6
  */
 #define OSTREE_REPO_METADATA_REF "ostree-metadata"
 
-#endif  /* OSTREE_ENABLE_EXPERIMENTAL_API */
+/**
+ * OSTREE_META_KEY_DEPLOY_COLLECTION_ID:
+ *
+ * GVariant type `s`. This key can be used in the repo metadata which is stored
+ * in OSTREE_REPO_METADATA_REF as well as in the summary. The semantics of this
+ * are that the remote repository wants clients to update their remote config
+ * to add this collection ID (clients can't do P2P operations involving a
+ * remote without a collection ID configured on it, even if one is configured
+ * on the server side). Clients must never change or remove a collection ID
+ * already set in their remote config.
+ *
+ * Currently, OSTree does not implement changing a remote config based on this
+ * key, but it may do so in a later release, and until then clients such as
+ * Flatpak may implement it.
+ *
+ * This is a replacement for the similar metadata key implemented by flatpak,
+ * `xa.collection-id`, which is now deprecated as clients which supported it had
+ * bugs with their P2P implementations.
+ *
+ * Since: 2018.9
+ */
+#define OSTREE_META_KEY_DEPLOY_COLLECTION_ID "ostree.deploy-collection-id"
 
 G_END_DECLS
 
